//Ref URL : https://www.shadertoy.com/view/MsXcWl regular
//#extension GL_OES_EGL_image_external : require
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;



// [-1;1] <=> [pink;orange]
const float EFFECT_AMOUNT = 0.5; // will make reds more pink

// Other parameters
const highp float SKIN_HUE = 0.05;
const highp float SKIN_HUE_TOLERANCE = 265.0;
//const float MAX_HUE_SHIFT = 0.04;
const highp float MAX_SATURATION_SHIFT = 0.8;
//vec3 color = vec3(0.0f,0.0f,0.0f);

// RGB <-> HSV conversion, thanks to http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
highp vec3 rgb2hsv(highp vec3 c)
{
    highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    highp float d = q.x - min(q.w, q.y);
    highp float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

//// HSV <-> RGB conversion, thanks to http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
highp vec3 hsv2rgb(highp vec3 c)
{
    highp vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

// Main
void main ()
{
    highp vec2 uv;
    uv = textureCoordinate * 1.0;

    // Sample the input pixel
    highp vec4 colorRGB = texture2D(inputImageTexture, uv);

    //    // get effect amount to apply
    highp float skin_tone_shift = EFFECT_AMOUNT;
    //
    //    // Convert color to HSV, extract hue
    highp vec3 colorHSV = rgb2hsv(colorRGB.rgb);
    highp float hue = colorHSV.x;
    highp float saturation = colorHSV.y;
    highp float value = colorHSV.z;


    if( (hue>=0.53 && hue<0.78) && saturation>0.2 && value>=0.07){//blue

        // check how far from skin hue
        highp float dist = hue - SKIN_HUE;
        if (dist > 0.5)
        dist -= 1.0;
        if (dist < -0.5)
        dist += 1.0;
        dist = abs(dist)/0.5; // normalized to [0,1]

        // Apply Gaussian like filter
        highp float weight = exp(-dist*dist*SKIN_HUE_TOLERANCE);
        weight = clamp(weight, 0.0, 1.0);

        colorHSV.y -= skin_tone_shift * weight * MAX_SATURATION_SHIFT;



        highp vec3 finalColorRGB = hsv2rgb(colorHSV.rgb);
        gl_FragColor = vec4(finalColorRGB, 1.0);
    }else{
        colorHSV.y = 0.0;
        //        colorHSV.z = 0.92;
        vec3 finalColorRGB = hsv2rgb(colorHSV.rgb);
        gl_FragColor = vec4(finalColorRGB, 1.0);
    }

}