precision mediump float;
varying vec2 vTexCoord;
uniform sampler2D uTexture;
uniform int colorMode;
uniform highp vec4 recolor;

float Lum(vec3 C){
  return 0.3 * C.r + 0.59 * C.g + 0.11 * C.b;
}

vec3 ClipColor (vec3 C){
  vec3 outColor = vec3(C.r, C.g, C.b);

  float L = Lum(C);

  float n = min(min(C.r, C.g), C.b);

  float x = max(max(C.r, C.g), C.b);

  if (n < 0.0) {
    outColor.r = L + ((C.r - L) * L) / (L - n);
    outColor.g = L + ((C.g - L) * L) / (L - n);
    outColor.b = L + ((C.b - L) * L) / (L - n);
  }

  if (x > 1.0) {
    outColor.r = L + ((C.r - L) * (1.0 - L)) / (x - L);
    outColor.g = L + ((C.g - L) * (1.0 - L)) / (x - L);
    outColor.b = L + ((C.b - L) * (1.0 - L)) / (x - L);
  }
  return outColor;
}

vec3 SetLum (vec3 C, float L){
  vec3 outColor = vec3(C.r, C.g, C.b);
  float d = L - Lum(C);

  outColor.r = outColor.r + d;
  outColor.g = outColor.g + d;
  outColor.b = outColor.b + d;

  return ClipColor(outColor);
}

vec3 Blend (vec3 Backdrop, vec3 Source) {
  return SetLum(Source, Lum(Backdrop));
}

void main() {
  vec4 color = texture2D(uTexture, vTexCoord);

  float L = Lum(color.rgb);
  vec4 finalColor = vec4(color.r, color.g, color.b, color.a);

  if(colorMode == 0) {
    float avg = 0.3 * color.r + 0.59 * color.g + 0.11 * color.b;
    finalColor.r = avg;
    finalColor.g = avg;
    finalColor.b = avg;
  }
  else if(colorMode == 2) {
    finalColor.rgb = SetLum(recolor.rgb, Lum(color.rgb));
  }
  else {
    finalColor.r = color.r * recolor.r;
    finalColor.g = color.g * recolor.g;
    finalColor.b = color.b * recolor.b;
  }

  gl_FragColor = finalColor;
}